@@ -101,10 +101,12 @@ class RpcClient:
             "tracingResult": {},
         }
         for event in frame.events:
-            # Use recurse=False so that each item in "sources" will not be iterated.
             event_dict = attr.asdict(
-                event, value_serializer=event.value_serializer, recurse=False
+                event,
+                filter=lambda field, _: field.name != "sources",
+                value_serializer=event.value_serializer,
             )
+
             # We have to explicitly write the type name because Js does not know it.
             event_dict["type"] = type(event).__name__
             frame_data["events"].append(event_dict)
