@@ -7,6 +7,7 @@ import sysconfig
 
 import argparse
 import cheap_repr
+import gc
 import inspect
 import jsonpickle
 import more_itertools
@@ -21,7 +22,7 @@ from pygments import highlight
 from pygments.formatters import Terminal256Formatter
 from pygments.lexers import PythonLexer
 from types import FrameType
-from typing import Any, Optional
+from typing import Any, Optional, Set
 
 from . import tracer
 
@@ -33,14 +34,14 @@ try:
     # Make sure Numpy and Pandas objects can be correctly encoded.
     # https://github.com/jsonpickle/jsonpickle#numpy-support
     jsonpickle_numpy.register_handlers()
-except ImportError:
+except (ImportError, RuntimeError):
     pass
 
 try:
     import jsonpickle.ext.pandas as jsonpickle_pandas
 
     jsonpickle_pandas.register_handlers()
-except ImportError:
+except (ImportError, RuntimeError):
     pass
 
 _INSTALLATION_PATHS = list(sysconfig.get_paths().values())
@@ -55,6 +56,33 @@ def repr_for_namespace(_, __):
     return "argparse.Namespace"
 
 
+def get_current_callable(frame: FrameType):
+    """Returns the callable that generates the frame.
+
+    See https://stackoverflow.com/a/52762678/2142577.
+    """
+    return [
+        obj
+        for obj in gc.get_referrers(frame.f_code)
+        if hasattr(obj, "__code__") and obj.__code__ is frame.f_code
+    ][0]
+
+
+def get_parameters(frame: FrameType) -> Set[str]:
+    """Get the parameters' names from a frame.
+
+    e.g. f(a, b, *args, **kwargs)
+    If called with f(1, 2, 3, x=1), returns {'a', 'b', 'args', 'kwargs'}.
+    """
+    arg_info = inspect.getargvalues(frame)
+    parameters = set(arg_info.args)
+    if arg_info.varargs is not None:
+        parameters.add(arg_info.varargs)
+    if arg_info.keywords is not None:
+        parameters.add(arg_info.keywords)
+    return parameters
+
+
 def should_ignore_event(
     *, target: str, value: Any, frame: Optional[FrameType] = None
 ) -> bool:
@@ -73,7 +101,7 @@ def should_ignore_event(
 
 
 def map_bytecode_offset_to_lineno(frame: FrameType) -> dict[int, int]:
-    """Maps bytecode offset to lineno in file.
+    """Maps bytecode offset to lineno in source code.
 
     Note that the lineno may not be accurate for multi-line statements. If we find
     this to be blocking, we might need to use a Range to represent lineno.
@@ -100,6 +128,12 @@ def get_jump_target_or_none(instr: dis.Instruction) -> Optional[int]:
 def to_json(python_object: Any):
     # TODO: Once we implemented better deserialization in Js, use unpicklable=True.
     try:
+        if (
+            hasattr(python_object, "__iter__")
+            and hasattr(python_object, "__next__")
+            and iter(python_object) == python_object
+        ):
+            raise Exception("Cannot encode iterators")
         json = jsonpickle.encode(python_object, unpicklable=False)
     except:
         # There are always things we just cannot encode, like a ML model.
@@ -178,16 +212,17 @@ def flatten(*args: any) -> list:
 
 
 def pprint(*args):
-    output = ""
-    for arg in args:
-        if isinstance(arg, str):
-            output += arg + "\n"
-        else:
+    output = "".join(
+        arg + "\n"
+        if isinstance(arg, str)
+        else (
             # Outputs syntax-highlighted object. See
             # https://gist.github.com/EdwardBetts/0814484fdf7bbf808f6f
-            output += (
-                highlight(pformat(arg), PythonLexer(), Terminal256Formatter()) + "\n"
-            )
+            highlight(pformat(arg), PythonLexer(), Terminal256Formatter())
+            + "\n"
+        )
+        for arg in args
+    )
     print(output)
 
 
@@ -201,6 +236,7 @@ def get_value_from_frame(name: str, frame: FrameType):
         return frame.f_builtins[name]
 
 
+# TODO: we can use a id(frame) -> name cache.
 def name_exist_in_frame(name: str, frame: FrameType) -> bool:
     return any(
         [name in frame.f_locals, name in frame.f_globals, name in frame.f_builtins]
